@@ -1,18 +1,20 @@
 import sys
+from typing import Any, Dict, Iterable, Optional
 
 import mlflow
 import pandas as pd
 import pytest
 import yaml
+from kedro.config import ConfigLoader
 from kedro.extras.datasets.pickle import PickleDataSet
-from kedro.framework.cli.utils import _add_src_to_path
-from kedro.framework.context import KedroContext
-from kedro.framework.project import configure_project
+from kedro.framework.hooks import hook_impl
+from kedro.framework.project import Validator, _ProjectPipelines, _ProjectSettings
 from kedro.framework.session import KedroSession
-from kedro.framework.startup import _get_project_metadata
+from kedro.framework.startup import bootstrap_project
 from kedro.io import DataCatalog, MemoryDataSet
 from kedro.pipeline import Pipeline, node
 from kedro.runner import SequentialRunner
+from kedro.versioning import Journal
 from mlflow.entities import RunStatus
 from mlflow.models import infer_signature
 from mlflow.tracking import MlflowClient
@@ -23,7 +25,11 @@ from kedro_mlflow.framework.hooks.pipeline_hook import (
     _format_conda_env,
     _generate_kedro_command,
 )
-from kedro_mlflow.io.metrics import MlflowMetricsDataSet
+from kedro_mlflow.io.metrics import (
+    MlflowMetricDataSet,
+    MlflowMetricHistoryDataSet,
+    MlflowMetricsDataSet,
+)
 from kedro_mlflow.pipeline import pipeline_ml_factory
 from kedro_mlflow.pipeline.pipeline_ml import PipelineML
 
@@ -91,6 +97,87 @@ def env_from_environment(environment_path, env_from_dict):
     return env_from_environment
 
 
+class DummyProjectHooks:
+    @hook_impl
+    def register_config_loader(self, conf_paths: Iterable[str]) -> ConfigLoader:
+        return ConfigLoader(conf_paths)
+
+    @hook_impl
+    def register_catalog(
+        self,
+        catalog: Optional[Dict[str, Dict[str, Any]]],
+        credentials: Dict[str, Dict[str, Any]],
+        load_versions: Dict[str, str],
+        save_version: str,
+        journal: Journal,
+    ) -> DataCatalog:
+        return DataCatalog.from_config(
+            catalog, credentials, load_versions, save_version, journal
+        )
+
+
+def _mock_imported_settings_paths(mocker, mock_settings):
+    for path in [
+        "kedro.framework.context.context.settings",
+        "kedro.framework.session.session.settings",
+        "kedro.framework.project.settings",
+    ]:
+        mocker.patch(path, mock_settings)
+    return mock_settings
+
+
+def _mock_settings_with_hooks(mocker, hooks):
+    class MockSettings(_ProjectSettings):
+        _HOOKS = Validator("HOOKS", default=hooks)
+
+    return _mock_imported_settings_paths(mocker, MockSettings())
+
+
+@pytest.fixture
+def mock_settings_with_mlflow_hooks(mocker):
+
+    return _mock_settings_with_hooks(
+        mocker,
+        hooks=(
+            DummyProjectHooks(),
+            MlflowPipelineHook(),
+            # MlflowNodeHook(),
+        ),
+    )
+
+
+@pytest.fixture(autouse=True)
+def mocked_logging(mocker):
+    # Disable logging.config.dictConfig in KedroSession._setup_logging as
+    # it changes logging.config and affects other unit tests
+    return mocker.patch("logging.config.dictConfig")
+
+
+@pytest.fixture
+def mock_failing_pipelines(mocker):
+    def failing_node():
+        mlflow.start_run(nested=True)
+        raise ValueError("Let's make this pipeline fail")
+
+    def mocked_register_pipelines():
+        failing_pipeline = Pipeline(
+            [
+                node(
+                    func=failing_node,
+                    inputs=None,
+                    outputs="fake_output",
+                )
+            ]
+        )
+        return {"__default__": failing_pipeline, "pipeline_off": failing_pipeline}
+
+    mocker.patch.object(
+        _ProjectPipelines,
+        "_get_pipelines_registry_callable",
+        return_value=mocked_register_pipelines,
+    )
+
+
 @pytest.mark.parametrize(
     "conda_env,expected",
     (
@@ -132,9 +219,15 @@ def dummy_pipeline():
     def train_fun(data, param):
         return 2
 
-    def metric_fun(data, model):
+    def metrics_fun(data, model):
         return {"metric_key": {"value": 1.1, "step": 0}}
 
+    def metric_fun(data, model):
+        return 1.1
+
+    def metric_history_fun(data, model):
+        return [0.1, 0.2]
+
     def predict_fun(model, data):
         return data * model
 
@@ -153,17 +246,41 @@ def dummy_pipeline():
                 tags=["training"],
             ),
             node(
-                func=metric_fun,
+                func=metrics_fun,
                 inputs=["model", "data"],
                 outputs="my_metrics",
                 tags=["training"],
             ),
             node(
-                func=metric_fun,
+                func=metrics_fun,
                 inputs=["model", "data"],
                 outputs="another_metrics",
                 tags=["training"],
             ),
+            node(
+                func=metric_fun,
+                inputs=["model", "data"],
+                outputs="my_metric",
+                tags=["training"],
+            ),
+            node(
+                func=metric_fun,
+                inputs=["model", "data"],
+                outputs="another_metric",
+                tags=["training"],
+            ),
+            node(
+                func=metric_history_fun,
+                inputs=["model", "data"],
+                outputs="my_metric_history",
+                tags=["training"],
+            ),
+            node(
+                func=metric_history_fun,
+                inputs=["model", "data"],
+                outputs="another_metric_history",
+                tags=["training"],
+            ),
             node(
                 func=predict_fun,
                 inputs=["model", "data"],
@@ -198,6 +315,10 @@ def dummy_catalog(tmp_path):
             "model": PickleDataSet((tmp_path / "model.csv").as_posix()),
             "my_metrics": MlflowMetricsDataSet(),
             "another_metrics": MlflowMetricsDataSet(prefix="foo"),
+            "my_metric": MlflowMetricDataSet(),
+            "another_metric": MlflowMetricDataSet(key="foo"),
+            "my_metric_history": MlflowMetricHistoryDataSet(),
+            "another_metric_history": MlflowMetricHistoryDataSet(key="bar"),
         }
     )
     return dummy_catalog
@@ -253,6 +374,7 @@ def pipeline_ml_with_parameters():
         training=full_pipeline.only_nodes_with_tags("training"),
         inference=full_pipeline.only_nodes_with_tags("inference"),
         input_name="data",
+        conda_env={"python": "3.7.0", "dependencies": ["kedro==0.16.5"]},
     )
     return pipeline_ml_with_parameters
 
@@ -298,13 +420,8 @@ def test_mlflow_pipeline_hook_with_different_pipeline_types(
     dummy_run_params,
 ):
 
-    project_metadata = _get_project_metadata(kedro_project_with_mlflow_conf)
-    _add_src_to_path(project_metadata.source_dir, kedro_project_with_mlflow_conf)
-    configure_project(project_metadata.package_name)
-    with KedroSession.create(
-        package_name=project_metadata.package_name,
-        project_path=kedro_project_with_mlflow_conf,
-    ):
+    bootstrap_project(kedro_project_with_mlflow_conf)
+    with KedroSession.create(project_path=kedro_project_with_mlflow_conf):
         # config_with_base_mlflow_conf is a conftest fixture
         pipeline_hook = MlflowPipelineHook()
         runner = SequentialRunner()
@@ -349,6 +466,8 @@ def test_mlflow_pipeline_hook_with_different_pipeline_types(
         # for metric
         assert dummy_catalog._data_sets["my_metrics"]._prefix == "my_metrics"
         assert dummy_catalog._data_sets["another_metrics"]._prefix == "foo"
+        assert dummy_catalog._data_sets["my_metric"].key == "my_metric"
+        assert dummy_catalog._data_sets["another_metric"].key == "foo"
 
         if isinstance(pipeline_to_run, PipelineML):
             trained_model = mlflow.pyfunc.load_model(f"runs:/{run_id}/model")
@@ -376,13 +495,8 @@ def test_mlflow_pipeline_hook_with_copy_mode(
     expected,
 ):
     # config_with_base_mlflow_conf is a conftest fixture
-    project_metadata = _get_project_metadata(kedro_project_with_mlflow_conf)
-    _add_src_to_path(project_metadata.source_dir, kedro_project_with_mlflow_conf)
-    configure_project(project_metadata.package_name)
-    with KedroSession.create(
-        package_name=project_metadata.package_name,
-        project_path=kedro_project_with_mlflow_conf,
-    ):
+    bootstrap_project(kedro_project_with_mlflow_conf)
+    with KedroSession.create(project_path=kedro_project_with_mlflow_conf):
 
         pipeline_hook = MlflowPipelineHook()
         runner = SequentialRunner()
@@ -403,7 +517,7 @@ def test_mlflow_pipeline_hook_with_copy_mode(
             training=dummy_pipeline_ml.training,
             inference=dummy_pipeline_ml.inference,
             input_name=dummy_pipeline_ml.input_name,
-            conda_env={},
+            conda_env={"python": "3.7.0", "dependencies": ["kedro==0.16.5"]},
             model_name=dummy_pipeline_ml.model_name,
             copy_mode=copy_mode,
         )
@@ -429,17 +543,12 @@ def test_mlflow_pipeline_hook_with_copy_mode(
         assert actual_copy_mode == expected
 
 
-def test_mlflow_pipeline_hook_metrics_with_run_id(
+def test_mlflow_pipeline_hook_metric_metrics_with_run_id(
     kedro_project_with_mlflow_conf, dummy_pipeline_ml, dummy_run_params
 ):
 
-    project_metadata = _get_project_metadata(kedro_project_with_mlflow_conf)
-    _add_src_to_path(project_metadata.source_dir, kedro_project_with_mlflow_conf)
-    configure_project(project_metadata.package_name)
-    with KedroSession.create(
-        package_name=project_metadata.package_name,
-        project_path=kedro_project_with_mlflow_conf,
-    ):
+    bootstrap_project(kedro_project_with_mlflow_conf)
+    with KedroSession.create(project_path=kedro_project_with_mlflow_conf):
 
         mlflow_conf = get_mlflow_config()
         mlflow.set_tracking_uri(mlflow_conf.mlflow_tracking_uri)
@@ -459,6 +568,14 @@ def test_mlflow_pipeline_hook_metrics_with_run_id(
                 "another_metrics": MlflowMetricsDataSet(
                     run_id=existing_run_id, prefix="foo"
                 ),
+                "my_metric": MlflowMetricDataSet(run_id=existing_run_id),
+                "another_metric": MlflowMetricDataSet(
+                    run_id=existing_run_id, key="foo"
+                ),
+                "my_metric_history": MlflowMetricHistoryDataSet(run_id=existing_run_id),
+                "another_metric_history": MlflowMetricHistoryDataSet(
+                    run_id=existing_run_id, key="bar"
+                ),
             }
         )
 
@@ -509,8 +626,17 @@ def test_mlflow_pipeline_hook_metrics_with_run_id(
         # Check if metrics datasets have prefix with its names.
         # for metric
         assert all_runs_id == {current_run_id, existing_run_id}
+
         assert run_data.metrics["my_metrics.metric_key"] == 1.1
         assert run_data.metrics["foo.metric_key"] == 1.1
+        assert run_data.metrics["my_metric"] == 1.1
+        assert run_data.metrics["foo"] == 1.1
+        assert (
+            run_data.metrics["my_metric_history"] == 0.2
+        )  # the list is tored, but only the last value is retrieved
+        assert (
+            run_data.metrics["bar"] == 0.2
+        )  # the list is tored, but only the last value is retrieved
 
 
 def test_mlflow_pipeline_hook_save_pipeline_ml_with_parameters(
@@ -520,13 +646,8 @@ def test_mlflow_pipeline_hook_save_pipeline_ml_with_parameters(
     dummy_run_params,
 ):
     # config_with_base_mlflow_conf is a conftest fixture
-    project_metadata = _get_project_metadata(kedro_project_with_mlflow_conf)
-    _add_src_to_path(project_metadata.source_dir, kedro_project_with_mlflow_conf)
-    configure_project(project_metadata.package_name)
-    with KedroSession.create(
-        package_name=project_metadata.package_name,
-        project_path=kedro_project_with_mlflow_conf,
-    ):
+    bootstrap_project(kedro_project_with_mlflow_conf)
+    with KedroSession.create(project_path=kedro_project_with_mlflow_conf):
 
         mlflow_conf = get_mlflow_config()
         mlflow.set_tracking_uri(mlflow_conf.mlflow_tracking_uri)
@@ -606,13 +727,8 @@ def test_mlflow_pipeline_hook_with_pipeline_ml_signature(
     expected_signature,
 ):
     # config_with_base_mlflow_conf is a conftest fixture
-    project_metadata = _get_project_metadata(kedro_project_with_mlflow_conf)
-    _add_src_to_path(project_metadata.source_dir, kedro_project_with_mlflow_conf)
-    configure_project(project_metadata.package_name)
-    with KedroSession.create(
-        package_name=project_metadata.package_name,
-        project_path=kedro_project_with_mlflow_conf,
-    ):
+    bootstrap_project(kedro_project_with_mlflow_conf)
+    with KedroSession.create(project_path=kedro_project_with_mlflow_conf):
         pipeline_hook = MlflowPipelineHook()
         runner = SequentialRunner()
 
@@ -690,45 +806,18 @@ def test_generate_default_kedro_commands(default_value):
     assert _generate_kedro_command(**record_data) == expected
 
 
-def test_on_pipeline_error(kedro_project_with_mlflow_conf):
+def test_on_pipeline_error(
+    kedro_project_with_mlflow_conf,
+    mock_settings_with_mlflow_hooks,
+    mock_failing_pipelines,
+):
 
     tracking_uri = (kedro_project_with_mlflow_conf / "mlruns").as_uri()
 
-    project_metadata = _get_project_metadata(kedro_project_with_mlflow_conf)
-    _add_src_to_path(project_metadata.source_dir, kedro_project_with_mlflow_conf)
-    configure_project(project_metadata.package_name)
-    with KedroSession.create(
-        package_name=project_metadata.package_name,
-        project_path=kedro_project_with_mlflow_conf,
-    ):
-
-        def failing_node():
-            mlflow.start_run(nested=True)
-            raise ValueError("Let's make this pipeline fail")
-
-        class DummyContextWithHook(KedroContext):
-            project_name = "fake project"
-            package_name = "fake_project"
-            project_version = "0.16.5"
-
-            hooks = (MlflowPipelineHook(),)
-
-            def _get_pipeline(self, name: str = None) -> Pipeline:
-                return Pipeline(
-                    [
-                        node(
-                            func=failing_node,
-                            inputs=None,
-                            outputs="fake_output",
-                        )
-                    ]
-                )
-
+    bootstrap_project(kedro_project_with_mlflow_conf)
+    with KedroSession.create(project_path=kedro_project_with_mlflow_conf) as session:
         with pytest.raises(ValueError):
-            failing_context = DummyContextWithHook(
-                "fake_package", kedro_project_with_mlflow_conf.as_posix()
-            )
-            failing_context.run()
+            session.run()
 
         # the run we want is the last one in Default experiment
         failing_run_info = MlflowClient(tracking_uri).list_run_infos("0")[0]
